/* * Copyright (c) 2024 Carl Klemm <carl@uvos.xyz>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or
 * other materials provided with the distribution.
 *  * Neither the name of %ORGANIZATION% nor the names of its contributors may be
 * used to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include <stdint.h>
#include <semaphore.h>
#include <stdbool.h>

/**
API to control a FanDevice.
* @defgroup API User API
* This API allows you to control a FanDevice.
* @{
*/

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	FAN_A = 0,
	FAN_B = 1,
	FAN_C = 2,
	FAN_D = 3,
	FAN_NONE = -1,
} channel_t;

enum {
	COMMAND_HEARTBEAT = 1,
	COMMAND_SET_FAN = 2,
};

struct fandevice {
	struct usbshm* priv;
};

/**
 * @brief Attempts to connect to a FanDevice and initializes a fandevice struct
 * @param device pointer to a FanDevice struct to initialize
 * @param serial The serial number of the device to connect to, or 0 for any
 * @return 0 on success and < 0 on failure
 */
int fandevice_connect(struct fandevice* device, uint16_t serial);

/**
 * @brief Sends a heartbeat package to the fan device to avoid it ramping up the fans
 * @param device pointer to a fandevice struct
 * @return 0 on success and < 0 on failure
 */
int fandevice_heartbeat(struct fandevice* device);

/**
 * @brief Connects the given channel(s) to the common inputs disconnecting all others
 * @param device pointer to a fandevice struct
 * @param channel A channel set the speed for.
 * @param speed the [0,1] to set the fan to.
 * @return 0 on success and < 0 on failure
 */
int fandevice_set_channel(struct fandevice* device, channel_t channel, float speed);

/**
 * @brief Disconnects from the fandevice
 */
void fandevice_disconnect(struct fandevice* device);

#ifdef __cplusplus
}
#endif

/**
....
* @}
*/
